@@ -2,7 +2,8 @@ from twisted.internet.protocol import Factory
 from twisted.protocols.amp import AMP, Command, Unicode, ListOf
 
 from bravo import version as bravo_version
-from bravo.ibravo import IConsoleCommand
+from bravo.factories.beta import BravoFactory
+from bravo.ibravo import IChatCommand, IConsoleCommand
 from bravo.plugin import retrieve_plugins
 
 class Version(Command):
@@ -47,6 +48,10 @@ class ConsoleRPCProtocol(AMP):
 
         # XXX hax
         self.commands = retrieve_plugins(IConsoleCommand)
+        # And chat commands, too.
+        chat = retrieve_plugins(IChatCommand)
+        for name, plugin in chat.iteritems():
+            self.commands[name] = IConsoleCommand(plugin)
         # Register aliases.
         for plugin in self.commands.values():
             for alias in plugin.aliases:
@@ -82,9 +87,15 @@ class ConsoleRPCFactory(Factory):
     protocol = ConsoleRPCProtocol
 
     def __init__(self, service):
-        self.factories = service.namedServices
+        self.services = service.namedServices
 
     def buildProtocol(self, addr):
-        protocol = self.protocol(self.factories)
+        factories = {}
+        for name, service in self.services.iteritems():
+            factory = service.args[1]
+            if isinstance(factory, BravoFactory):
+                factories[factory.name] = factory
+
+        protocol = self.protocol(factories)
         protocol.factory = self
         return protocol
