@@ -33,35 +33,26 @@ class Block(object):
     def __init__(self, slot, name, drop=None, replace=0, ratio=1,
             quantity=1, dim=16, breakable=True, orientation=None):
         """
-        A block in a chunk.
-
-        :Parameters:
-            slot : int
-                The index of this block. Must be globally unique.
-            name : str
-                A common name for this block.
-            drop : int
-                The type of block that should be dropped when an instance of
-                this block is destroyed. Defaults to the slot value, to drop
-                instances of this same type of block. To indicate that this
-                block does not drop anything, set to air.
-            replace : int
-                The type of block to place in the map when instances of this
-                block are destroyed. Defaults to air.
-            ratio : float
-                The probability of this block dropping a block on destruction.
-            quantity : int
-                The number of blocks dropped when this block is destroyed.
-            dim : int
-                How much light dims when passing through this kind of block.
-                Defaults to 16 = opaque block.
-            breakable : bool
-                Whether this block is diggable, breakable, bombable,
-                explodeable, etc. Only a few blocks actually genuinely cannot
-                be broken, so the default is True.
-            orientation : tuple
-                The orientation data for a block. See ``orientable()`` for an
-                explanation. The data should be in standard face order.
+        :param int slot: The index of this block. Must be globally unique.
+        :param str name: A common name for this block.
+        :param int drop: The type of block that should be dropped when an
+            instance of this block is destroyed. Defaults to the slot value, to
+            drop instances of this same type of block. To indicate that this
+            block does not drop anything, set to air.
+        :param int replace: The type of block to place in the map when
+            instances of this block are destroyed. Defaults to air.
+        :param float ratio: The probability of this block dropping a block
+            on destruction.
+        :param int quantity: The number of blocks dropped when this block
+            is destroyed.
+        :param int dim: How much light dims when passing through this kind
+            of block. Defaults to 16 = opaque block.
+        :param bool breakable: Whether this block is diggable, breakable,
+            bombable, explodeable, etc. Only a few blocks actually genuinely
+            cannot be broken, so the default is True.
+        :param tuple orientation: The orientation data for a block. See
+            :meth:`orientable` for an explanation. The data should be in standard
+            face order.
         """
 
         self.slot = slot
@@ -122,6 +113,9 @@ class Block(object):
         determine the face against which they were built.
 
         Ladders are orientable, signposts are not.
+
+        :rtype: bool
+        :returns: True if this block can be oriented, False if not.
         """
 
         return any(self._o_dict)
@@ -132,7 +126,7 @@ class Block(object):
         cannot be built against the given face.
 
         This method only returns valid data for orientable blocks; check
-        ``orientable()`` first.
+        :meth:`orientable` first.
         """
 
         return self._o_dict.get(face)
@@ -149,9 +143,6 @@ class Item(object):
     )
 
     def __init__(self, slot, name):
-        """
-        An item.
-        """
 
         self.slot = slot
         self.name = name
@@ -253,6 +244,9 @@ block_names = [
     "portal",
     "jack-o-lantern",
     "cake",
+    "redstone-repeater-off",
+    "redstone-repeater-on",
+    "locked-chest",
 ]
 
 item_names = [
@@ -355,6 +349,9 @@ item_names = [
     "bone",
     "sugar",
     "cake",
+    "bed",
+    "redstone-repeater",
+    "cookie",
 ]
 
 special_item_names = [
@@ -369,12 +366,14 @@ drops = {}
 drops[1]  = 4   # Stone           -> Cobblestone
 drops[2]  = 3   # Grass           -> Dirt
 drops[20] = 0   # Glass
+drops[52] = 0   # Mob spawner
 drops[60] = 3   # Soil            -> Dirt
 drops[62] = 61  # Burning Furnace -> Furnace
 drops[78] = 0   # Snow
 
 # Block -> item drops.
 drops[16] = 263 # Coal Ore Block    -> Coal
+drops[26] = 355 # Bed block         -> Bed
 drops[56] = 264 # Diamond Ore Block -> Diamond
 drops[63] = 323 # Sign Post         -> Sign Item
 drops[64] = 324 # Wooden Door       -> Wooden Door Item
@@ -382,6 +381,9 @@ drops[68] = 323 # Wall Sign         -> Sign Item
 drops[71] = 330 # Iron Door         -> Iron Door Item
 drops[83] = 338 # Reed              -> Reed Item
 drops[89] = 348 # Lightstone        -> Lightstone Dust
+drops[93] = 356 # Redstone-repeater-on  -> Redstone-repeater
+drops[94] = 356 # Redstone-repeater-off -> Redstone-repeater
+
 
 unbreakables = set()
 
@@ -390,6 +392,44 @@ unbreakables.add(7)  # Bedrock
 unbreakables.add(10) # Lava
 unbreakables.add(11) # Lava spring
 
+dims = {}
+
+dims[0]  = 0 # Air
+dims[6]  = 0 # Sapling
+dims[10] = 0 # Lava
+dims[11] = 0 # Lava spring
+dims[20] = 0 # Glass
+dims[26] = 0 # Bed
+dims[37] = 0 # Yellow Flowers
+dims[38] = 0 # Red Flowers
+dims[39] = 0 # Brown Mushrooms
+dims[40] = 0 # Red Mushrooms
+dims[44] = 0 # Single Step
+dims[51] = 0 # Fire
+dims[52] = 0 # Mob spawner
+dims[53] = 0 # Wooden stairs
+dims[55] = 0 # Redstone (Wire)
+dims[59] = 0 # Crops
+dims[60] = 0 # Soil
+dims[63] = 0 # Sign
+dims[64] = 0 # Wood door
+dims[66] = 0 # Rails
+dims[67] = 0 # Stone stairs
+dims[68] = 0 # Sign (on wall)
+dims[69] = 0 # Lever
+dims[70] = 0 # Stone Pressure Plate
+dims[71] = 0 # Iron door
+dims[72] = 0 # Wood Pressure Plate
+dims[78] = 0 # Snow
+dims[81] = 0 # Cactus
+dims[83] = 0 # Sugar Cane
+dims[85] = 0 # Fence
+dims[90] = 0 # Portal
+dims[92] = 0 # Cake
+dims[93] = 0 # redstone-repeater-off
+dims[94] = 0 # redstone-repeater-on
+
+
 blocks = {}
 """
 A dictionary of ``Block`` objects.
@@ -412,18 +452,19 @@ _add_block(Block(13, "gravel", drop=318, ratio=1 / 10))
 # Leaves drop saplings, with 1 in 9 odds, and dims by 1.
 _add_block(Block(18, "leaves", drop=6, ratio=1 / 9, dim=1))
 # Torches are orientable.
-_add_block(Block(50, "torch", orientation=(None, 5, 4, 3, 2, 1)))
+_add_block(Block(50, "torch", orientation=(None, 5, 4, 3, 2, 1), dim=0))
 # Furnaces are orientable.
 _add_block(Block(61, "furnace", orientation=(0, 1, 2, 3, 4, 5)))
 # Ladders are orientable.
-_add_block(Block(65, "ladder", orientation=(None, None, 2, 3, 4, 5)))
+_add_block(Block(65, "ladder", orientation=(None, None, 2, 3, 4, 5), dim=0))
 # Redstone ore drops 5 redstone dusts.
 _add_block(Block(73, "redstone-ore", drop=331, quantity=5))
 _add_block(Block(74, "glowing-redstone-ore", drop=331, quantity=5))
 # Redstone torches are orientable.
-_add_block(Block(76, "redstone-torch", orientation=(None, 5, 4, 3, 2, 1)))
+_add_block(Block(75, "redstone-torch-off", orientation=(None, 5, 4, 3, 2, 1), dim=0))
+_add_block(Block(76, "redstone-torch", orientation=(None, 5, 4, 3, 2, 1), dim=0))
 # Stone buttons are orientable.
-_add_block(Block(77, "stone-button", orientation=(None, None, 1, 2, 3, 4)))
+_add_block(Block(77, "stone-button", orientation=(None, None, 1, 2, 3, 4), dim=0))
 # Ice drops nothing, is replaced by springs, and dims by 3.
 _add_block(Block(79, "ice", drop=0, replace=8, dim=3))
 # Clay drops 4 clay balls.
@@ -449,6 +490,9 @@ for i, name in enumerate(block_names):
         kwargs["drop"] = drops[i]
     if i in unbreakables:
         kwargs["breakable"] = False
+    if i in dims:
+        kwargs["dim"] = dims[i]
+
     b = Block(i, name, **kwargs)
     _add_block(b)
 
@@ -468,10 +512,20 @@ for i, name in enumerate(special_item_names):
 
 glowing_blocks = {
     blocks["torch"].slot: 14,
+    blocks["lightstone"].slot: 15,
+    blocks["jack-o-lantern"].slot: 15,
+    blocks["fire"].slot: 15,
+    blocks["lava"].slot: 15,
+    blocks["lava-spring"].slot: 15,
+    blocks["locked-chest"].slot: 15,
+    blocks["burning-furnace"].slot: 13,
+    blocks["portal"].slot: 11,
+    blocks["glowing-redstone-ore"].slot: 9,
+    blocks["redstone-repeater-on"].slot: 9,
+    blocks["redstone-torch"].slot: 7,
+    blocks["brown-mushroom"].slot: 1,
 }
 
-blocks["air"].dim = 0
-
 armor_helmets = (86, 298, 302, 306, 310, 314)
 """
 List of slots of helmets.
