@@ -4,7 +4,7 @@ from twisted.python import log
 
 from bravo.inventory import Equipment, ChestStorage
 from bravo.location import Location
-from bravo.packets import make_packet
+from bravo.packets.beta import make_packet
 
 class Entity(object):
     """
@@ -104,6 +104,35 @@ class Player(Entity):
             )
         return packet
 
+class Painting(Entity):
+    """
+    A painting on a wall.
+    """
+
+    name = "Painting"
+
+    def __init__(self, direction=1, motive="", **kwargs):
+        """
+        Create a painting.
+
+        This method calls super().
+        """
+
+        super(Painting, self).__init__(**kwargs)
+
+        self.direction = direction
+        self.motive = motive
+
+    def save_to_packet(self):
+        return make_packet("painting",
+            eid=self.eid,
+            title=self.motive,
+            x=self.location.x,
+            y=self.location.y,
+            z=self.location.z,
+            direction=self.direction
+        )
+
 class Pickup(Entity):
     """
     Class representing a dropped block or item.
@@ -140,10 +169,148 @@ class Pickup(Entity):
             roll=0
         )
 
+class Mob(Entity):
+    """
+    A creature.
+    """
+
+    name = "Mob"
+    type = "mob"
+
+    metadata = {0: ("byte", 0)}
+
+    def save_to_packet(self):
+        return make_packet("mob",
+            eid=self.eid,
+            type=self.type,
+            x=self.location.x * 32 + 16,
+            y=self.location.y * 32,
+            z=self.location.z * 32 + 16,
+            yaw=0,
+            pitch=0,
+            metadata=self.metadata
+        )
+
+class Chuck(Mob):
+    """
+    A cross between a duck and a chicken.
+    """
+
+    name = "Chicken"
+    type = "chuck"
+
+class Cow(Mob):
+    """
+    Large, four-legged milk containers.
+    """
+
+    name = "Cow"
+    type = "cow"
+
+class Pig(Mob):
+    """
+    A provider of bacon and piggyback rides.
+    """
+
+    name = "Pig"
+    type = "pig"
+
+    metadata = {0: ("byte", 0), 16: ("byte", 0)}
+
+class Sheep(Mob):
+    """
+    A wooly mob.
+    """
+
+    name = "Sheep"
+
+    metadata = {0: ("byte", 0), 16: ("byte", 0)}
+
+    def __init__(self, sheared=False, color=0, **kwargs):
+        """
+        Create a sheep.
+
+        This method calls super().
+        """
+
+        super(Sheep, self).__init__(**kwargs)
+
+        self.sheared = sheared
+        self.color = color
+
+    def save_to_packet(self):
+        # Prepare metadata.
+        metadata = self.metadata.copy()
+        color = self.color
+        if self.sheared:
+            color |= 0x10
+        metadata[16] = "byte", color
+
+        return make_packet("mob",
+            eid=self.eid,
+            type="sheep",
+            x=self.location.x * 32 + 16,
+            y=self.location.y * 32,
+            z=self.location.z * 32 + 16,
+            yaw=0,
+            pitch=0,
+            metadata=self.metadata
+        )
+
+class Slime(Mob):
+    """
+    A gelatinous blob.
+    """
+
+    name = "Slime"
+
+    metadata = {0: ("byte", 0), 16: ("byte", 0)}
+
+    def __init__(self, size=1, **kwargs):
+        """
+        Create a slime.
+
+        This method calls super().
+        """
+
+        super(Sheep, self).__init__(**kwargs)
+
+        self.size = size
+
+    def save_to_packet(self):
+        # Prepare metadata.
+        metadata = self.metadata.copy()
+        metadata[16] = "byte", self.size
+
+        return make_packet("mob",
+            eid=self.eid,
+            type="slime",
+            x=self.location.x * 32 + 16,
+            y=self.location.y * 32,
+            z=self.location.z * 32 + 16,
+            yaw=0,
+            pitch=0,
+            metadata=self.metadata
+        )
+
+class Squid(Mob):
+    """
+    An aquatic source of ink.
+    """
+
+    name = "Squid"
+    type = "squid"
+
 entities = dict((entity.name, entity)
     for entity in (
-        Player,
+        Chuck,
+        Cow,
+        Painting,
         Pickup,
+        Pig,
+        Player,
+        Sheep,
+        Squid,
     )
 )
 
@@ -202,6 +369,13 @@ class MobSpawner(Tile):
 
     name = "MobSpawner"
 
+class Music(Tile):
+    """
+    A tile which produces a pitch when whacked.
+    """
+
+    name = "Music"
+
 class Sign(Tile):
     """
     A tile that stores text.
@@ -238,6 +412,7 @@ tiles = dict((tile.name, tile)
         Chest,
         Furnace,
         MobSpawner,
+        Music,
         Sign,
     )
 )
