@@ -10,7 +10,7 @@ from bravo.config import configuration
 from bravo.entity import entities
 from bravo.ibravo import IAuthenticator, ISeason, ITerrainGenerator
 from bravo.location import Location
-from bravo.packets import make_packet
+from bravo.packets.beta import make_packet
 from bravo.plugin import retrieve_named_plugins, retrieve_sorted_plugins
 from bravo.protocols.beta import BannedProtocol, BravoProtocol
 from bravo.utilities import chat_name, sanitize_chat
@@ -131,10 +131,9 @@ class BravoFactory(Factory):
         bigx = entity.location.x // 16
         bigz = entity.location.z // 16
 
-        chunk = self.world.load_chunk(bigx, bigz)
-        chunk.entities.add(entity)
-
-        log.msg("Created entity %s" % entity)
+        d = self.world.request_chunk(bigx, bigz)
+        d.addCallback(lambda chunk: chunk.entities.add(entity))
+        d.addCallback(lambda none: log.msg("Created entity %s" % entity))
 
         return entity
 
@@ -163,10 +162,9 @@ class BravoFactory(Factory):
         bigx = entity.location.x // 16
         bigz = entity.location.z // 16
 
-        chunk = self.world.load_chunk(bigx, bigz)
-        chunk.entities.discard(entity)
-
-        log.msg("Destroyed entity %s" % entity)
+        d = self.world.request_chunk(bigx, bigz)
+        d.addCallback(lambda chunk: chunk.entities.discard(entity))
+        d.addCallback(lambda none: log.msg("Destroyed entity %s" % entity))
 
     def update_time(self):
         """
@@ -296,6 +294,18 @@ class BravoFactory(Factory):
 
             quantity -= 64
 
+    def players_near(self, player, radius):
+        """
+        Obtain other players within a radius of a given player.
+
+        Radius is measured in blocks.
+        """
+
+        for i in (p for p in self.protocols.itervalues()
+            if player.location.distance(p.location) <= radius and
+            p.player != player):
+            yield i.player
+
     def stopFactory(self):
         """
         Called before factory stops listening on ports. Used to perform
