@@ -1,5 +1,11 @@
 from twisted.plugin import IPlugin
-from zope.interface import Attribute
+from twisted.python.components import registerAdapter
+from zope.interface import implements, invariant, Attribute
+
+class InvariantException(Exception):
+    """
+    Exception raised by failed invariant conditions.
+    """
 
 class IBravoPlugin(IPlugin):
     """
@@ -10,6 +16,9 @@ class IBravoPlugin(IPlugin):
 
     name = Attribute("""
         The name of the plugin.
+
+        This name is used to reference the plugin in configurations, and also
+        to uniquely index the plugin.
         """)
 
 class ISortedPlugin(IBravoPlugin):
@@ -17,15 +26,19 @@ class ISortedPlugin(IBravoPlugin):
     Parent interface for sorted plugins.
 
     Sorted plugins have an innate and automatic ordering inside lists thanks
-    to dependency lists.
+    to the ability to advertise their dependencies.
     """
 
     before = Attribute("""
         Plugins which must come before this plugin in the pipeline.
+
+        Should be a tuple, list, or some other iterable.
         """)
 
     after = Attribute("""
         Plugins which must come after this plugin in the pipeline.
+
+        Should be a tuple, list, or some other iterable.
         """)
 
 class IAuthenticator(IBravoPlugin):
@@ -120,11 +133,45 @@ class IConsoleCommand(ICommand):
         :returns: a generator object or other iterable yielding lines
         """
 
+class ChatToConsole(object):
+    """
+    Adapt a chat command to be used on the console.
+
+    This largely consists of passing the username correctly.
+    """
+
+    implements(IConsoleCommand)
+
+    def __init__(self, chatcommand):
+        self.chatcommand = chatcommand
+
+        self.aliases = self.chatcommand.aliases
+        self.info = self.chatcommand.info
+        self.name = self.chatcommand.name
+        self.usage = "<username> %s" % self.chatcommand.usage
+
+    def console_command(self, factory, parameters):
+        if IConsoleCommand.providedBy(self.chatcommand):
+            return self.chatcommand.console_command(factory, parameters)
+        else:
+            username = parameters.pop(0) if parameters else ""
+            return self.chatcommand.chat_command(factory, username,
+                parameters)
+
+registerAdapter(ChatToConsole, IChatCommand, IConsoleCommand)
+
+def recipe_invariant(r):
+    # Size invariant.
+    if len(r.recipe) != r.dimensions[0] * r.dimensions[1]:
+        raise InvariantException("Recipe size is invalid")
+
 class IRecipe(IBravoPlugin):
     """
     Recipe for crafting materials from other materials.
     """
 
+    invariant(recipe_invariant)
+
     dimensions = Attribute("""
         Tuple representing the size of the recipe.
         """)
@@ -172,41 +219,57 @@ class ISerializer(IBravoPlugin):
     def save_chunk(chunk):
         """
         Save a chunk.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def load_chunk(chunk):
         """
         Load a chunk.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def save_level(level):
         """
         Save a level.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def load_level(level):
         """
         Load a level.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def save_player(player):
         """
         Save a player.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def load_player(player):
         """
         Load a player.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def load_plugin_data(name):
         """
-        Load data of a plugin.
+        Load plugin-specific data.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
     def save_plugin_data(name, value):
         """
-        Save data of a plugin.
+        Save plugin-specific data.
+
+        May return a ``Deferred`` that will fire on completion.
         """
 
 class ISerializerFactory(IBravoPlugin):
@@ -243,16 +306,30 @@ class IBuildHook(ISortedPlugin):
         of building should continue after this hook runs. Use it to halt build
         hook processing, if needed.
 
+        For sanity purposes, build hooks may return a ``Deferred`` which will
+        fire with their return values, but are not obligated to do so.
+
         A trivial do-nothing build hook looks like the following:
 
         >>> def build_hook(self, factory, player, builddata):
         ...     return True, builddata
 
+        To make life more pleasant when returning deferred values, use
+        ``inlineCallbacks``, which many of the standard build hooks use:
+
+        >>> @inlineCallbacks
+        ... def build_hook(self, factory, player, builddata):
+        ...     returnValue((True, builddata))
+
+        This form makes it much easier to deal with asynchronous operations on
+        the factory and world.
+
         :param ``Factory`` factory: factory
         :param ``Player`` player: player entity doing the building
         :param namedtuple builddata: permanent building location and data
 
-        :returns: tuple of build data and whether subsequent hooks will run
+        :returns: ``Deferred`` with tuple of build data and whether subsequent
+                  hooks will run
         """
 
 class IDigHook(ISortedPlugin):
@@ -291,3 +368,25 @@ class ISignHook(ISortedPlugin):
         :param list text: list of lines of text
         :param bool new: whether this sign is newly placed
         """
+
+class IUseHook(ISortedPlugin):
+    """
+    Hook for actions to be taken when a player interacts with an entity.
+
+    Each plugin needs to specify a list of entity types it is interested in
+    in advance, and it will only be called for those.
+    """
+
+    def use_hook(factory, player, target, alternate):
+        """
+        Do things.
+
+        :param `Factory` factory: factory
+        :param `Player` player: player
+        :param `Entity` target: target of the interaction
+        :param bool alternate: whether the player right-clicked the target
+        """
+
+    targets = Attribute("""
+        List of entity names this plugin wants to be called for.
+        """)
