@@ -1,12 +1,14 @@
-from zope.interface import implements
+from textwrap import wrap
+
 from twisted.internet import reactor
+from zope.interface import implements
 
 from bravo.blocks import blocks, items
 from bravo.config import configuration
 from bravo.ibravo import IChatCommand, IConsoleCommand, ISeason
 from bravo.plugin import retrieve_plugins, retrieve_named_plugins
 from bravo.plugin import PluginException
-from bravo.packets import make_packet
+from bravo.packets.beta import make_packet
 from bravo.utilities import split_time
 
 def parse_player(factory, name):
@@ -21,10 +23,13 @@ def parse_block(block):
     """
 
     try:
-        if block.startswith("0x"):
+        if block.startswith("0x") and (
+            (int(block, 16) in blocks) or (int(block, 16) in items)):
             return (int(block, 16), 0)
-        else:
+        elif (int(block) in blocks) or (int(block) in items):
             return (int(block), 0)
+        else:
+            raise Exception("Couldn't find block id %s!" % block)
     except ValueError:
         if block in blocks:
             return blocks[block].key
@@ -43,36 +48,57 @@ class Help(object):
 
     implements(IChatCommand, IConsoleCommand)
 
-    def dispatch(self, plugins):
-        l = []
+    def general_help(self, plugins):
+        """
+        Return a list of commands.
+        """
 
-        # This is fairly brute-force and inelegant. I'm very open to
-        # suggestions on improving it.
-        for plugin in set(plugins.itervalues()):
-            usage = "%s %s" % (plugin.name, plugin.usage)
-            l.append((plugin.name, usage, plugin.info))
-            for alias in plugin.aliases:
-                alias_usage = usage.replace(plugin.name, alias)
-                info = "Alias for %s" % plugin.name
-                l.append((alias, alias_usage, info))
+        commands = [plugin.name for plugin in set(plugins.itervalues())]
+        commands.sort()
 
-        name_pad = max(len(i[0]) for i in l) + 1
-        usage_pad = max(len(i[1]) for i in l) + 1
+        wrapped = wrap(", ".join(commands), 60)
 
-        yield "%s %s %s" % ("Name:".ljust(name_pad),
-            "Usage:".ljust(usage_pad), "Info:")
+        help_text = [
+            "Use /help <command> for more information on a command.",
+            "List of commands:",
+        ] + wrapped
 
-        for name, usage, info in sorted(l):
-            yield "%s %s %s" % (name.ljust(name_pad), usage.ljust(usage_pad),
-                info)
+        return help_text
+
+    def specific_help(self, plugins, name):
+        """
+        Return specific help about a single plugin.
+        """
+
+        try:
+            plugin = plugins[name]
+        except:
+            return ("No such command!",)
+
+        help_text = [
+            "Usage: %s %s" % (plugin.name, plugin.usage),
+        ]
+
+        if plugin.aliases:
+            help_text.append("Aliases: %s" % ", ".join(plugin.aliases))
+
+        help_text.append(plugin.info)
+
+        return help_text
 
     def chat_command(self, factory, username, parameters):
-        for i in self.dispatch(retrieve_plugins(IChatCommand)):
-            yield i
+        if parameters:
+            return self.specific_help(retrieve_plugins(IChatCommand),
+                "".join(parameters))
+        else:
+            return self.general_help(retrieve_plugins(IChatCommand))
 
     def console_command(self, factory, parameters):
-        for i in self.dispatch(retrieve_plugins(IConsoleCommand)):
-            yield i
+        if parameters:
+            return self.specific_help(retrieve_plugins(IConsoleCommand),
+                "".join(parameters))
+        else:
+            return self.general_help(retrieve_plugins(IConsoleCommand))
 
     name = "help"
     aliases = tuple()
@@ -155,7 +181,7 @@ class Say(object):
 
 class Give(object):
 
-    implements(IChatCommand, IConsoleCommand)
+    implements(IChatCommand)
 
     def chat_command(self, factory, username, parameters):
         if len(parameters) == 0:
@@ -185,10 +211,6 @@ class Give(object):
         # Return an empty tuple for iteration
         return tuple()
 
-    def console_command(self, factory, parameters):
-        for i in self.chat_command(factory, parameters[0], parameters[1:]):
-            yield i
-
     name = "give"
     aliases = tuple()
     usage = "<block> <quantity>"
@@ -306,21 +328,18 @@ class Season(object):
     info = "Advance date to the beginning of the given season"
 
 class Me(object):
+
     implements(IChatCommand)
 
-    def dispatch(self, username, says):
-        say = " ".join(says)
-        msg = "* %s %s" % (username,say)
-        yield msg
-
     def chat_command(self, factory, username, parameters):
-        for i in self.dispatch(username, parameters):
-            yield i
+        say = " ".join(parameters)
+        msg = "* %s %s" % (username, say)
+        return (msg,)
 
     name = "me"
     aliases = tuple()
     usage = "<message>"
-    info = "/me like IRC"
+    info = "Emote"
 
 class Kick(object):
     """
