@@ -1,5 +1,7 @@
 from zope.interface import implements
 
+from twisted.internet.defer import inlineCallbacks, returnValue
+
 from bravo.blocks import blocks
 from bravo.ibravo import IBuildHook, IDigHook
 from bravo.utilities import split_coords
@@ -40,13 +42,13 @@ class Fallables(object):
 
         chunk.set_column(x, z, column)
 
-
+    @inlineCallbacks
     def build_hook(self, factory, player, builddata):
         bigx, smallx, bigz, smallz = split_coords(builddata.x, builddata.z)
-        chunk = factory.world.load_chunk(bigx, bigz)
+        chunk = yield factory.world.request_chunk(bigx, bigz)
         self.dig_hook(factory, chunk, smallx, builddata.y, smallz,
             builddata.block)
-        return True, builddata
+        returnValue((True, builddata))
 
     name = "fallables"
 
