@@ -1,6 +1,7 @@
 from collections import defaultdict
 from itertools import chain, product
 
+from twisted.internet.defer import inlineCallbacks
 from twisted.internet.task import LoopingCall
 from zope.interface import implements
 
@@ -47,7 +48,9 @@ class Fluid(object):
         Load all of the important blocks in the chunk into memory.
         """
 
+    @inlineCallbacks
     def process(self):
+
         for factory in self.pending:
             w = factory.world
             new = set()
@@ -59,7 +62,7 @@ class Fluid(object):
                 # Our downstairs pal.
                 below = (x, y - 1, z)
 
-                block = w.get_block((x, y, z))
+                block = yield w.get_block((x, y, z))
                 if block == self.sponge:
                     # Track this sponge.
                     self.sponges[factory][x, y, z] = True
@@ -70,7 +73,7 @@ class Fluid(object):
                         xrange(max(y - 2, 0), min(y + 3, 128)),
                         xrange(z - 2, z + 3),
                         ):
-                        target = w.get_block(coords)
+                        target = yield w.get_block(coords)
                         if target == self.spring:
                             if (coords[0], coords[2]) in self.springs[factory]:
                                 del self.springs[factory][coords[0],
@@ -101,14 +104,16 @@ class Fluid(object):
 
                     # Spawn water from springs.
                     for coords in neighbors:
-                        if (w.get_block(coords) in self.whitespace and
+                        neighbor = yield w.get_block(coords)
+                        if (neighbor in self.whitespace and
                             not any(self.sponges[factory].iteritemsnear(coords, 2))):
                             w.set_block(coords, self.fluid)
                             w.set_metadata(coords, 0x0)
                             new.add(coords)
 
                     # Is this water falling down to the next y-level?
-                    if (y > 0 and w.get_block(below) in self.whitespace and
+                    neighbor = yield w.get_block(below)
+                    if (y > 0 and neighbor in self.whitespace and
                         not any(self.sponges[factory].iteritemsnear(below, 2))):
                         w.set_block(below, self.fluid)
                         w.set_metadata(below, FALLING)
@@ -124,7 +129,7 @@ class Fluid(object):
                     # Let's see if there are any springs nearby which are
                     # above us and thus able to fuel us.
                     if not any(springy >= y
-                        for springy in self.springs[factory].iterkeysnear(
+                        for springy in self.springs[factory].itervaluesnear(
                             (x, z), self.levels + 1
                         )
                     ):
@@ -138,13 +143,14 @@ class Fluid(object):
                     newmd = self.levels + 1
 
                     for coords in neighbors:
-                        jones = w.get_block(coords)
+                        jones = yield w.get_block(coords)
                         if jones == self.spring:
                             newmd = 0
                             new.update(neighbors)
                             break
                         elif jones == self.fluid:
-                            jonesmd = w.get_metadata(coords) & ~FALLING
+                            jonesmd = yield w.get_metadata(coords)
+                            jonesmd &= ~FALLING
                             if jonesmd + 1 < newmd:
                                 newmd = jonesmd + 1
 
@@ -159,7 +165,8 @@ class Fluid(object):
                     # will only mark lower water levels than ourselves, and
                     # only if they are definitely too low.
                     for coords in neighbors:
-                        if w.get_metadata(coords) & ~FALLING > newmd + 1:
+                        neighbor = yield w.get_metadata(coords)
+                        if neighbor & ~FALLING > newmd + 1:
                             new.add(coords)
 
                     # Now, it's time to extend water. Remember, either the
@@ -167,7 +174,8 @@ class Fluid(object):
                     # flows out across the xz-level, but *not* both.
 
                     # Fall down to the next y-level, if possible.
-                    if (y > 0 and w.get_block(below) in self.whitespace and
+                    neighbor = yield w.get_block(below)
+                    if (y > 0 and neighbor in self.whitespace and
                         not any(self.sponges[factory].iteritemsnear(below, 2))):
                         w.set_block(below, self.fluid)
                         w.set_metadata(below, newmd | FALLING)
@@ -183,7 +191,8 @@ class Fluid(object):
                     if newmd < self.levels:
                         newmd += 1
                         for coords in neighbors:
-                            if (w.get_block(coords) in self.whitespace and
+                            neighbor = yield w.get_block(coords)
+                            if (neighbor in self.whitespace and
                                 not any(self.sponges[factory].iteritemsnear(coords, 2))):
                                 w.set_block(coords, self.fluid)
                                 w.set_metadata(coords, newmd)
@@ -215,10 +224,11 @@ class Fluid(object):
             # Flush affected chunks.
             to_flush = defaultdict(set)
             for x, y, z in chain(self.pending[factory], new):
-                to_flush[factory].add(factory.world.load_chunk(x // 16, z // 16))
-            for factory, chunks in to_flush.iteritems():
-                for chunk in chunks:
-                    factory.flush_chunk(chunk)
+                to_flush[factory].add((x // 16, z // 16))
+            for factory, coords in to_flush.iteritems():
+                for x, z in coords:
+                    d = factory.world.request_chunk(x, z)
+                    d.addCallback(factory.flush_chunk)
 
             self.pending[factory] = new
 
@@ -238,7 +248,6 @@ class Fluid(object):
         """
 
         block, metadata, x, y, z, face = builddata
-
         # No, you may not place this. Only I may place it.
         if block.slot == self.fluid:
             factory.world.destroy((x, y, z))
@@ -251,6 +260,7 @@ class Fluid(object):
 
         return True, builddata
 
+    @inlineCallbacks
     def dig_hook(self, factory, chunk, x, y, z, block):
         """
         Check for neighboring water that might want to spread.
@@ -281,7 +291,8 @@ class Fluid(object):
                 ( 1, 0,  0),
                 (-1, 0,  0)):
                 coords = x + dx, y + dy, z + dz
-                if factory.world.get_block(coords) in (self.spring, self.fluid):
+                test_block = yield factory.world.get_block(coords)
+                if test_block in (self.spring, self.fluid):
                     self.pending[factory].add(coords)
 
         if any(self.pending.itervalues()) and not self.loop.running:
@@ -333,6 +344,7 @@ class Redstone(object):
 
         self.loop = LoopingCall(self.process)
 
+    @inlineCallbacks
     def update_wires(self, factory, x, y, z, enabled):
         """
         Trace the wires starting at a certain point, and either enable or
@@ -346,7 +358,7 @@ class Redstone(object):
         while traveling:
             # Visit nodes.
             for coords in traveling:
-                metadata = factory.world.get_metadata(coords)
+                metadata = yield factory.world.get_metadata(coords)
                 if metadata != level:
                     factory.world.set_metadata(coords, level)
                     traveled.add(coords)
@@ -362,9 +374,10 @@ class Redstone(object):
                 for (i, j, k) in traveling]
             traveling.clear()
             for l in nodes:
-                traveling.update(coords for coords in l
-                if factory.world.get_block(coords) ==
-                    blocks["redstone-wire"].slot)
+                for coords in l:
+                    block = yield factory.world.get_block(coords)
+                    if block == blocks["redstone-wire"].slot:
+                        traveling.add(coords)
             traveling.difference_update(traveled)
 
             if level:
