@@ -106,17 +106,32 @@ class Alpha(object):
                 raise Exception("Could not create world in %s" % self.folder)
 
         self._entity_loaders = {
+            "Chicken": lambda entity, tag: None,
+            "Cow": lambda entity, tag: None,
             "Item": self._load_item_from_tag,
+            "Painting": self._load_painting_from_tag,
+            "Pig": lambda entity, tag: None,
+            "Sheep": self._load_sheep_from_tag,
+            "Slime": self._load_slime_from_tag,
+            "Squid": lambda entity, tag: None,
         }
 
         self._entity_savers = {
+            "Chicken": lambda entity, tag: None,
+            "Cow": lambda entity, tag: None,
             "Item": self._save_item_to_tag,
+            "Painting": self._save_painting_to_tag,
+            "Pig": lambda entity, tag: None,
+            "Sheep": self._save_sheep_to_tag,
+            "Slime": self._save_slime_to_tag,
+            "Squid": lambda entity, tag: None,
         }
 
         self._tile_loaders = {
             "Chest": self._load_chest_from_tag,
             "Furnace": self._load_furnace_from_tag,
             "MobSpawner": self._load_mobspawner_from_tag,
+            "Music": self._load_music_from_tag,
             "Sign": self._load_sign_from_tag,
         }
 
@@ -124,6 +139,7 @@ class Alpha(object):
             "Chest": self._save_chest_to_tag,
             "Furnace": self._save_furnace_to_tag,
             "MobSpawner": self._save_mobspawner_to_tag,
+            "Music": self._save_music_to_tag,
             "Sign": self._save_sign_to_tag,
         }
 
@@ -188,6 +204,37 @@ class Alpha(object):
         tag["Item"]["Damage"] = TAG_Short(item.item[1])
         tag["Item"]["Count"] = TAG_Short(item.quantity)
 
+    def _load_painting_from_tag(self, painting, tag):
+        painting.direction = tag["Dir"].value
+        painting.motive = tag["Motive"].value
+        # Overwrite position with absolute block coordinates of image's
+        # center. Original position seems to be unused.
+        painting.location.x = tag["TileX"].value
+        painting.location.y = tag["TileY"].value
+        painting.location.z = tag["TileZ"].value
+
+    def _save_painting_to_tag(self, painting, tag):
+        tag["Dir"] = TAG_Byte(painting.direction)
+        tag["Motive"] = TAG_String(painting.motive)
+        # Both tile and position will be the center of the image.
+        tag["TileX"] = TAG_Int(painting.location.x)
+        tag["TileY"] = TAG_Int(painting.location.y)
+        tag["TileZ"] = TAG_Int(painting.location.z)
+
+    def _load_sheep_from_tag(self, sheep, tag):
+        sheep.sheared = bool(tag["Sheared"].value)
+        sheep.color = tag["Color"].value
+
+    def _save_sheep_to_tag(self, sheep, tag):
+        tag["Sheared"] = TAG_Byte(sheep.sheared)
+        tag["Color"] = TAG_Byte(sheep.color)
+
+    def _load_slime_from_tag(self, slime, tag):
+        slime.size = tag["Size"].value
+
+    def _save_slime_to_tag(self, slime, tag):
+        tag["Size"] = TAG_Byte(slime.size)
+
     # Tile serializers. Tiles are blocks and entities at the same time, in the
     # worst way. Each of these helpers will be called during chunk serialize
     # and deserialize automatically; they never need to be called directly.
@@ -247,6 +294,12 @@ class Alpha(object):
         tag["EntityId"] = TAG_String(ms.mob)
         tag["Delay"] = TAG_Short(ms.delay)
 
+    def _load_music_from_tag(self, music, tag):
+        music.note = tag["note"].value
+
+    def _save_music_to_tag(self, music, tag):
+        tag["Music"] = TAG_Byte(music.note)
+
     def _load_sign_from_tag(self, sign, tag):
         sign.text1 = tag["Text1"].value
         sign.text2 = tag["Text2"].value
@@ -303,6 +356,8 @@ class Alpha(object):
                     chunk.tiles[tile.x, tile.y, tile.z] = tile
                 except KeyError:
                     print "Unknown tile entity %s" % tag["id"].value
+                    print "Tag for tile:"
+                    print tag.pretty_tree()
 
         chunk.dirty = not chunk.populated
 
