@@ -1,12 +1,12 @@
-from collections import namedtuple
-from itertools import product
+from collections import namedtuple, defaultdict
+from itertools import product, chain
 from time import time
 from urlparse import urlunparse
 from math import pi
 
 from twisted.internet import reactor
-from twisted.internet.defer import succeed
-from twisted.internet.endpoints import TCP4ClientEndpoint
+from twisted.internet.defer import (DeferredList, inlineCallbacks,
+    maybeDeferred, succeed)
 from twisted.internet.protocol import Protocol
 from twisted.internet.task import cooperate, LoopingCall
 from twisted.internet.task import TaskDone, TaskFailed
@@ -16,17 +16,18 @@ from twisted.web.client import getPage
 from bravo.blocks import blocks, items
 from bravo.config import configuration
 from bravo.entity import Sign
+from bravo.motd import get_motd
 from bravo.factories.infini import InfiniClientFactory
-from bravo.ibravo import IChatCommand, IBuildHook, IDigHook, ISignHook
+from bravo.ibravo import IChatCommand, IBuildHook, IDigHook, ISignHook, IUseHook
 from bravo.inventory import Workbench, sync_inventories
 from bravo.location import Location
-from bravo.packets import parse_packets, make_packet, make_error_packet
-from bravo.plugin import retrieve_plugins, retrieve_sorted_plugins
+from bravo.packets.beta import parse_packets, make_packet, make_error_packet
+from bravo.plugin import retrieve_plugins, retrieve_sorted_plugins, retrieve_named_plugins
 from bravo.utilities import split_coords
 
 (STATE_UNAUTHENTICATED, STATE_CHALLENGED, STATE_AUTHENTICATED) = range(3)
 
-SUPPORTED_PROTOCOL = 10
+SUPPORTED_PROTOCOL = 11
 
 circle = [(i, j)
     for i, j in product(xrange(-10, 10), xrange(-10, 10))
@@ -70,6 +71,7 @@ class BetaServerProtocol(Protocol):
             1: self.login,
             2: self.handshake,
             3: self.chat,
+            7: self.use,
             10: self.grounded,
             11: self.position,
             12: self.orientation,
@@ -127,6 +129,11 @@ class BetaServerProtocol(Protocol):
         Hook for chat packets.
         """
 
+    def use(self, container):
+        """
+        Hook for use packets.
+        """
+
     def grounded(self, container):
         """
         Hook for grounded packets.
@@ -353,8 +360,8 @@ class BetaProxyProtocol(BetaServerProtocol):
     def login(self, container):
         self.username = container.username
 
-        packet = make_packet("login", protocol=0, username="", unused="",
-            seed=0, dimension=0)
+        packet = make_packet("login", protocol=0, username="", seed=0,
+            dimension=0)
         self.transport.write(packet)
 
         url = urlunparse(("http", self.gateway, "/node/0/0/", None, None,
@@ -373,6 +380,8 @@ class BetaProxyProtocol(BetaServerProtocol):
         Add a new node to this client.
         """
 
+        from twisted.internet.endpoints import TCP4ClientEndpoint
+
         log.msg("Adding node %s:%d" % (address, port))
 
         endpoint = TCP4ClientEndpoint(reactor, address, port, 5)
@@ -420,13 +429,24 @@ class BravoProtocol(BetaServerProtocol):
             [])
         self.sign_hooks = retrieve_sorted_plugins(ISignHook, names)
 
+        names = configuration.getlistdefault(self.config_name, "use_hooks",
+            [])
+        self.use_hooks = defaultdict(list)
+        for plugin in retrieve_named_plugins(IUseHook, names):
+            for target in plugin.targets:
+                self.use_hooks[target].append(plugin)
+
+        # Retrieve the MOTD. Only needs to be done once.
+        self.motd = configuration.getdefault(self.config_name, "motd", None)
+
         self.last_dig_build_timer = time()
 
+    @inlineCallbacks
     def authenticated(self):
         BetaServerProtocol.authenticated(self)
 
         # Init player, and copy data into it.
-        self.player = self.factory.world.load_player(self.username)
+        self.player = yield self.factory.world.load_player(self.username)
         self.player.eid = self.eid
         self.location = self.player.location
 
@@ -446,19 +466,16 @@ class BravoProtocol(BetaServerProtocol):
         # ourselves.
         for protocol in self.factory.protocols.itervalues():
             packet = protocol.player.save_to_packet()
-            self.transport.write(packet)
-            packet = protocol.player.save_equipment_to_packet()
-            self.transport.write(packet)
-            packet = make_packet("create", eid=protocol.player.eid)
+            packet += protocol.player.save_equipment_to_packet()
+            packet += make_packet("create", eid=protocol.player.eid)
             self.transport.write(packet)
 
         self.factory.protocols[self.username] = self
 
+        # Send spawn and inventory.
         spawn = self.factory.world.spawn
         packet = make_packet("spawn", x=spawn[0], y=spawn[1], z=spawn[2])
-        self.transport.write(packet)
-
-        packet = self.player.inventory.save_to_packet()
+        packet += self.player.inventory.save_to_packet()
         self.transport.write(packet)
 
         self.send_initial_chunk_and_location()
@@ -594,6 +611,20 @@ class BravoProtocol(BetaServerProtocol):
             message = "<%s> %s" % (self.username, container.message)
             self.factory.chat(message)
 
+    def use(self, container):
+        """
+        For each entity in proximity (4 blocks), check if it is the target
+        of this packet and call all hooks that stated interested in this
+        type.
+        """
+        nearby_players = self.factory.players_near(self.player, 4)
+        for entity in chain(self.entities_near(4), nearby_players):
+            if entity.eid == container.target:
+                for hook in self.use_hooks[entity.name]:
+                    hook.use_hook(self.factory, self.player, entity,
+                        container.button == 0)
+                break
+
     def digging(self, container):
         # XXX several improvements should happen here
         # ~ We should time started and stopped pairs to force clients to
@@ -651,12 +682,15 @@ class BravoProtocol(BetaServerProtocol):
 
         oldblock = blocks[chunk.get_block((smallx, container.y, smallz))]
 
+        l = []
         for hook in self.dig_hooks:
-            hook.dig_hook(self.factory, chunk, smallx, container.y, smallz,
-                oldblock)
+            l.append(maybeDeferred(hook.dig_hook,
+                self.factory, chunk, smallx, container.y, smallz, oldblock))
 
-        self.factory.flush_chunk(chunk)
+        dl = DeferredList(l)
+        dl.addCallback(lambda none: self.factory.flush_chunk(chunk))
 
+    @inlineCallbacks
     def build(self, container):
         if container.x == -1 and container.z == -1 and container.y == 255:
             # Lala-land build packet. Discard it for now.
@@ -712,8 +746,8 @@ class BravoProtocol(BetaServerProtocol):
             container.z, container.face)
 
         for hook in self.build_hooks:
-            cont, builddata = hook.build_hook(self.factory, self.player,
-                builddata)
+            cont, builddata = yield maybeDeferred(hook.build_hook,
+                self.factory, self.player, builddata)
             if not cont:
                 break
 
@@ -900,9 +934,6 @@ class BravoProtocol(BetaServerProtocol):
             packet = entity.save_to_packet()
             self.transport.write(packet)
 
-            packet = make_packet("create", eid=entity.eid)
-            self.transport.write(packet)
-
         for entity in chunk.tiles.itervalues():
             if entity.name == "Sign":
                 packet = entity.save_to_packet()
@@ -929,6 +960,14 @@ class BravoProtocol(BetaServerProtocol):
         # spawned.
         d.addCallback(lambda none: self.update_location())
         d.addCallback(lambda none: self.position_changed())
+
+        # Send the MOTD.
+        if self.motd:
+            packet = make_packet("chat",
+                message=self.motd.replace("<tagline>", get_motd()))
+            d.addCallback(lambda none: self.transport.write(packet))
+
+        # Finally, start the secondary chunk loop.
         d.addCallback(lambda none: self.update_chunks())
 
     def update_location(self):
