@@ -21,8 +21,8 @@ try:
     import termios
     import tty
     fancy_console = os.isatty(sys.__stdin__.fileno())
-    fancy_console = fancy_console and configuration.getboolean("bravo",
-        "fancy_console")
+    fancy_console = fancy_console and configuration.getbooleandefault("bravo",
+        "fancy_console", True)
 except ImportError:
     fancy_console = False
 
@@ -43,7 +43,7 @@ class AMPGateway(object):
     Wrapper around the logical implementation of a console.
     """
 
-    def __init__(self, host, port):
+    def __init__(self, host, port=25600):
         self.ready = False
 
         self.host = host
@@ -91,6 +91,11 @@ class AMPGateway(object):
                 # Quit.
                 stop_console()
                 reactor.stop()
+            elif command == "worlds":
+                # Print list of available worlds.
+                self.sendLine("Worlds:")
+                for world in self.worlds:
+                    self.sendLine(world)
             elif command == "select":
                 # World selection.
                 world = params[0]
