@@ -1,8 +1,24 @@
-import unittest
+from twisted.trial import unittest
 
 import bravo.blocks
 import bravo.inventory
 
+from bravo.inventory import Slot
+
+class TestSlot(unittest.TestCase):
+    """
+    Double-check a few things about ``Slot``.
+    """
+
+    def test_decrement_none(self):
+        slot = Slot(0, 0, 1)
+        self.assertEqual(slot.decrement(), None)
+
+    def test_holds(self):
+        slot1 = Slot(4, 5, 1)
+        slot2 = Slot(4, 5, 1)
+        self.assertTrue(slot1.holds(slot2))
+
 class TestInventoryInternals(unittest.TestCase):
     """
     The Inventory class might be near-useless when not subclassed, but we can
@@ -48,54 +64,130 @@ class TestEquipmentInternals(unittest.TestCase):
         self.assertEqual(self.i.holdables[0], (2, 0, 2))
         self.assertEqual(self.i.holdables[1], None)
 
+    def test_add_to_inventory_fill_slot(self):
+        self.i.holdables[0] = Slot(2, 0, 50)
+        self.assertTrue(self.i.add((2, 0), 30))
+        self.assertEqual(self.i.holdables[0], (2, 0, 64))
+        self.assertEqual(self.i.holdables[1], (2, 0, 16))
+
     def test_consume_holdable(self):
-        self.i.holdables[0] = (2, 0, 1)
-        self.assertTrue(self.i.consume((2, 0)))
+        self.i.holdables[0] = Slot(2, 0, 1)
+        self.assertTrue(self.i.consume((2, 0), 0))
         self.assertEqual(self.i.holdables[0], None)
 
     def test_consume_holdable_empty(self):
-        self.assertFalse(self.i.consume((2, 0)))
+        self.assertFalse(self.i.consume((2, 0), 0))
 
     def test_consume_holdable_second_slot(self):
-        self.i.holdables[1] = (2, 0, 1)
-        self.assertTrue(self.i.consume((2, 0)))
+        self.i.holdables[1] = Slot(2, 0, 1)
+        self.assertTrue(self.i.consume((2, 0), 1))
+        self.assertEqual(self.i.holdables[1], None)
+
+    def test_consume_holdable_multiple_stacks(self):
+        self.i.holdables[0] = Slot(2, 0, 1)
+        self.i.holdables[1] = Slot(2, 0, 1)
+        # consume second stack
+        self.assertTrue(self.i.consume((2, 0), 1))
+        self.assertEqual(self.i.holdables[0], (2, 0, 1))
+        self.assertEqual(self.i.holdables[1], None)
+        # consume second stack a second time
+        self.assertFalse(self.i.consume((2, 0), 1))
+        self.assertEqual(self.i.holdables[0], (2, 0, 1))
         self.assertEqual(self.i.holdables[1], None)
 
     def test_select_stack(self):
-        self.i.holdables[0] = (2, 0, 1)
-        self.i.holdables[1] = (2, 0, 1)
+        self.i.holdables[0] = Slot(2, 0, 1)
+        self.i.holdables[1] = Slot(2, 0, 1)
         self.i.select(37)
         self.i.select(36)
         self.assertEqual(self.i.holdables[0], (2, 0, 2))
         self.assertEqual(self.i.holdables[1], None)
 
     def test_select_switch(self):
-        self.i.holdables[0] = (2, 0, 1)
-        self.i.holdables[1] = (3, 0, 1)
+        self.i.holdables[0] = Slot(2, 0, 1)
+        self.i.holdables[1] = Slot(3, 0, 1)
         self.i.select(36)
         self.i.select(37)
         self.i.select(36)
         self.assertEqual(self.i.holdables[0], (3, 0, 1))
         self.assertEqual(self.i.holdables[1], (2, 0, 1))
 
+    def test_select_secondary_switch(self):
+        self.i.holdables[0] = Slot(2, 0, 1)
+        self.i.holdables[1] = Slot(3, 0, 1)
+        self.i.select(36)
+        self.i.select(37, True)
+        self.i.select(36, True)
+        self.assertEqual(self.i.holdables[0], (3, 0, 1))
+        self.assertEqual(self.i.holdables[1], (2, 0, 1))
+
+    def test_select_outside_window(self):
+        self.assertFalse(self.i.select(64537))
+
     def test_select_secondary(self):
-        self.i.holdables[0] = (2, 0, 4)
+        self.i.holdables[0] = Slot(2, 0, 4)
         self.i.select(36, True)
         self.assertEqual(self.i.holdables[0], (2, 0, 2))
         self.assertEqual(self.i.selected, (2, 0, 2))
 
+    def test_select_secondary_empty(self):
+        for i in range(0, 45):
+            self.assertFalse(self.i.select(i, True))
+
+    def test_select_secondary_outside_window(self):
+        """
+        Test that outrageous selections, such as those generated by clicking
+        outside inventory windows, fail cleanly.
+        """
+
+        self.assertFalse(self.i.select(64537), True)
+
     def test_select_secondary_selected(self):
-        self.i.selected = (2, 0, 2)
+        self.i.selected = Slot(2, 0, 2)
         self.i.select(36, True)
         self.assertEqual(self.i.holdables[0], (2, 0, 1))
         self.assertEqual(self.i.selected, (2, 0, 1))
 
     def test_select_secondary_odd(self):
-        self.i.holdables[0] = (2, 0, 3)
+        self.i.holdables[0] = Slot(2, 0, 3)
         self.i.select(36, True)
         self.assertEqual(self.i.holdables[0], (2, 0, 1))
         self.assertEqual(self.i.selected, (2, 0, 2))
 
+    def test_select_fill_up_stack(self):
+        # create two stacks
+        self.i.holdables[0] = Slot(2, 0, 40)
+        self.i.holdables[1] = Slot(2, 0, 30)
+        # select first one
+        self.i.select(36)
+        # first slot is now empty - holding 40 items
+        self.assertEqual(self.i.selected, (2, 0, 40))
+        # second stack is untouched
+        self.assertEqual(self.i.holdables[1], (2, 0, 30))
+        # select second stack with right click
+        self.i.select(37)
+        # sums up to more than 64 items - fill up the second stack
+        self.assertEqual(self.i.holdables[1], (2, 0, 64))
+        # still hold the left overs
+        self.assertEqual(self.i.selected, (2, 0, 6))
+
+    def test_select_secondary_fill_up_stack(self):
+        # create two stacks
+        self.i.holdables[0] = Slot(2, 0, 40)
+        self.i.holdables[1] = Slot(2, 0, 30)
+        # select first one
+        self.i.select(36)
+        # first slot is now empty - holding 40 items
+        self.assertEqual(self.i.selected, (2, 0, 40))
+        # second stack is untouched
+        self.assertEqual(self.i.holdables[1], (2, 0, 30))
+        # select second stack with left click
+        self.i.select(37, True)
+        # sums up to more than 64 items - fill up the second stack
+        self.assertEqual(self.i.holdables[1], (2, 0, 64))
+        # still hold the left overs
+        self.assertEqual(self.i.selected, (2, 0, 6))
+
 class TestCraftingWood(unittest.TestCase):
     """
     Test basic crafting functionality.
@@ -112,7 +204,7 @@ class TestCraftingWood(unittest.TestCase):
         pass
 
     def test_check_crafting(self):
-        self.i.crafting[0] = (bravo.blocks.blocks["log"].slot, 0, 1)
+        self.i.crafting[0] = Slot(bravo.blocks.blocks["log"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(2)
         self.assertTrue(self.i.recipe)
@@ -121,7 +213,7 @@ class TestCraftingWood(unittest.TestCase):
             (bravo.blocks.blocks["wood"].slot, 0, 4))
 
     def test_check_crafting_multiple(self):
-        self.i.crafting[0] = (bravo.blocks.blocks["log"].slot, 0, 2)
+        self.i.crafting[0] = Slot(bravo.blocks.blocks["log"].slot, 0, 2)
         # Force crafting table to be rechecked.
         self.i.select(2)
         # Only checking count of crafted table; the previous test assured that
@@ -130,7 +222,7 @@ class TestCraftingWood(unittest.TestCase):
             (bravo.blocks.blocks["wood"].slot, 0, 4))
 
     def test_check_crafting_offset(self):
-        self.i.crafting[1] = (bravo.blocks.blocks["log"].slot, 0, 1)
+        self.i.crafting[1] = Slot(bravo.blocks.blocks["log"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(1)
         self.assertTrue(self.i.recipe)
@@ -152,8 +244,8 @@ class TestCraftingSticks(unittest.TestCase):
         pass
 
     def test_check_crafting(self):
-        self.i.crafting[0] = (bravo.blocks.blocks["wood"].slot, 0, 1)
-        self.i.crafting[2] = (bravo.blocks.blocks["wood"].slot, 0, 1)
+        self.i.crafting[0] = Slot(bravo.blocks.blocks["wood"].slot, 0, 1)
+        self.i.crafting[2] = Slot(bravo.blocks.blocks["wood"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(2)
         self.assertTrue(self.i.recipe)
@@ -162,8 +254,8 @@ class TestCraftingSticks(unittest.TestCase):
             (bravo.blocks.items["stick"].slot, 0, 4))
 
     def test_check_crafting_multiple(self):
-        self.i.crafting[0] = (bravo.blocks.blocks["wood"].slot, 0, 2)
-        self.i.crafting[2] = (bravo.blocks.blocks["wood"].slot, 0, 2)
+        self.i.crafting[0] = Slot(bravo.blocks.blocks["wood"].slot, 0, 2)
+        self.i.crafting[2] = Slot(bravo.blocks.blocks["wood"].slot, 0, 2)
         # Force crafting table to be rechecked.
         self.i.select(2)
         # Only checking count of crafted table; the previous test assured that
@@ -172,8 +264,8 @@ class TestCraftingSticks(unittest.TestCase):
             (bravo.blocks.items["stick"].slot, 0, 4))
 
     def test_check_crafting_offset(self):
-        self.i.crafting[1] = (bravo.blocks.blocks["wood"].slot, 0, 1)
-        self.i.crafting[3] = (bravo.blocks.blocks["wood"].slot, 0, 1)
+        self.i.crafting[1] = Slot(bravo.blocks.blocks["wood"].slot, 0, 1)
+        self.i.crafting[3] = Slot(bravo.blocks.blocks["wood"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(1)
         self.assertTrue(self.i.recipe)
@@ -194,8 +286,8 @@ class TestCraftingTorches(unittest.TestCase):
         pass
 
     def test_check_crafting(self):
-        self.i.crafting[0] = (bravo.blocks.items["coal"].slot, 0, 1)
-        self.i.crafting[2] = (bravo.blocks.items["stick"].slot, 0, 1)
+        self.i.crafting[0] = Slot(bravo.blocks.items["coal"].slot, 0, 1)
+        self.i.crafting[2] = Slot(bravo.blocks.items["stick"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(2)
         self.assertTrue(self.i.recipe)
@@ -204,8 +296,8 @@ class TestCraftingTorches(unittest.TestCase):
             (bravo.blocks.blocks["torch"].slot, 0, 4))
 
     def test_check_crafting_multiple(self):
-        self.i.crafting[0] = (bravo.blocks.items["coal"].slot, 0, 2)
-        self.i.crafting[2] = (bravo.blocks.items["stick"].slot, 0, 2)
+        self.i.crafting[0] = Slot(bravo.blocks.items["coal"].slot, 0, 2)
+        self.i.crafting[2] = Slot(bravo.blocks.items["stick"].slot, 0, 2)
         # Force crafting table to be rechecked.
         self.i.select(2)
         # Only checking count of crafted table; the previous test assured that
@@ -214,8 +306,8 @@ class TestCraftingTorches(unittest.TestCase):
             (bravo.blocks.blocks["torch"].slot, 0, 4))
 
     def test_check_crafting_offset(self):
-        self.i.crafting[1] = (bravo.blocks.items["coal"].slot, 0, 1)
-        self.i.crafting[3] = (bravo.blocks.items["stick"].slot, 0, 1)
+        self.i.crafting[1] = Slot(bravo.blocks.items["coal"].slot, 0, 1)
+        self.i.crafting[3] = Slot(bravo.blocks.items["stick"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(1)
         self.assertTrue(self.i.recipe)
@@ -237,14 +329,14 @@ class TestCraftingFurnace(unittest.TestCase):
         pass
 
     def test_check_crafting(self):
-        self.i.crafting[0] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[1] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[2] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[3] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[5] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[6] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[7] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
-        self.i.crafting[8] = (bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[0] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[1] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[2] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[3] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[5] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[6] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[7] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
+        self.i.crafting[8] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 1)
         # Force crafting table to be rechecked.
         self.i.select(5)
         self.assertTrue(self.i.recipe)
@@ -253,14 +345,14 @@ class TestCraftingFurnace(unittest.TestCase):
             (bravo.blocks.blocks["furnace"].slot, 0, 1))
 
     def test_check_crafting_multiple(self):
-        self.i.crafting[0] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[1] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[2] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[3] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[5] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[6] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[7] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
-        self.i.crafting[8] = (bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[0] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[1] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[2] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[3] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[5] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[6] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[7] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
+        self.i.crafting[8] = Slot(bravo.blocks.blocks["cobblestone"].slot, 0, 2)
         # Force crafting table to be rechecked.
         self.i.select(5)
         self.assertEqual(self.i.crafted[0],
@@ -340,6 +432,51 @@ class TestInventoryIntegration(unittest.TestCase):
         self.assertEqual(self.i.crafting[0], None)
         self.assertEqual(self.i.crafted[0], None)
 
+    def test_armor_slots_take_one_item_only(self):
+        self.i.add((bravo.blocks.items["iron-helmet"].slot, 0), 5)
+        self.i.select(36)
+        self.i.select(5)
+        self.assertEqual(self.i.armor[0], (bravo.blocks.items["iron-helmet"].slot, 0, 1))
+        self.assertEqual(self.i.selected, (bravo.blocks.items["iron-helmet"].slot, 0, 4))
+
+        # Exchanging one iron-helmet in the armor slot against 5 gold-helmet in the hand
+        # is not possible.
+        self.i.add((bravo.blocks.items["gold-helmet"].slot, 0), 5)
+        self.i.select(36)
+        self.i.select(5)
+        self.assertEqual(self.i.armor[0], (bravo.blocks.items["iron-helmet"].slot, 0, 1))
+        self.assertEqual(self.i.selected, (bravo.blocks.items["gold-helmet"].slot, 0, 5))
+
+    def test_armor_slots_take_armor_items_only(self):
+        self.i.add((bravo.blocks.blocks["dirt"].slot, 0), 10)
+        self.i.select(36)
+        self.assertFalse(self.i.select(5))
+        self.assertEqual(self.i.armor[0], None)
+        self.assertEqual(self.i.selected, (bravo.blocks.blocks["dirt"].slot, 0, 10))
+
+    def test_pumpkin_as_helmet(self):
+        self.i.add((bravo.blocks.blocks["pumpkin"].slot, 0), 1)
+        self.i.select(36)
+        self.i.select(5)
+        self.assertEqual(self.i.armor[0], (bravo.blocks.blocks["pumpkin"].slot, 0, 1))
+        self.assertEqual(self.i.selected, None)
+
+    def test_armor_only_in_matching_slots(self):
+        for index, item in enumerate(["leather-helmet", "chainmail-chestplate",
+                                      "diamond-leggings", "gold-boots"]):
+            self.i.add((bravo.blocks.items[item].slot, 0), 1)
+            self.i.select(36)
+
+            # Can't be placed in other armor slots.
+            other_slots = list(range(4))
+            other_slots.remove(index)
+            for i in other_slots:
+                self.assertFalse(self.i.select(5 + i))
+
+            # But it can in the appropriate slot.
+            self.assertTrue(self.i.select(5 + index))
+            self.assertEqual(self.i.armor[index], (bravo.blocks.items[item].slot, 0, 1))
+
 class TestWorkbenchIntegration(unittest.TestCase):
     """
     select() numbers
@@ -351,7 +488,7 @@ class TestWorkbenchIntegration(unittest.TestCase):
 
     def setUp(self):
         self.i = bravo.inventory.Workbench()
-    
+
     def test_trivial(self):
         pass
 
